function [aStatus aMessage]=SonnetCallEm(theFilename,theOptions)
% SIMULATE will call Sonnet EM
% to simulate the Sonnet Project File.
%
% This function has the following
% parameters:
%   1) (Optional) a string
%       that defines extra options
%       in the format explained below.
%
% Options are passed as a single
% string. Order of option switches doesnt
% matter and unknown option switches are
% ignored.
%
% Supported option switches:
%   '-c'          to clean the project data first
%   '-x'          to not clean the project data first (default)
%   '-w'          to display a simulation status window (default)
%   '-t'          to not display a simulation status window
%   '-r'          to run the simulation instantaneously (default)
%   '-p'          to not run the simulation instantaneously (requires the status window)
%   '-u'          to unlock the project (default)
%   '-l'          to not unlock the project
%   '-v <VERSION> to use a particular version of sonnet to do the simulation
%
%
% SonnetLab, all included documentation, all included examples
% and all other files (unless otherwise specified) are copyrighted by Sonnet Software
% in 2010 with all rights reserved.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS". ANY AND
% ALL EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED. UNDER NO CIRCUMSTANCES AND UNDER
% NO LEGAL THEORY, TORT, CONTRACT, OR OTHERWISE, SHALL THE COPYWRITE HOLDERS,  CONTRIBUTORS,
% MATLAB, OR SONNET SOFTWARE BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR
% CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF
% GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL
% DAMAGES OR LOSSES, OR FOR ANY DAMAGES EVEN IF THE COPYWRITE HOLDERS, CONTRIBUTORS, MATLAB,
% OR SONNET SOFTWARE HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES, OR FOR ANY CLAIM
% BY ANY OTHER PARTY.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Get the Sonnet Path
% If they specified a specific version of Sonnet to use then use that version
if ~isempty(findstr('-v',theOptions)) || ~isempty(findstr('-V',theOptions))
    
    % Parse the input string
    [aParcialString, aRemainingString] = strtok(theOptions,'vV'); %#ok<ASGLU>
    
    % Read in the version number
    aRemainingString=aRemainingString(2:length(aRemainingString));
    aSonnetVersion=sscanf(aRemainingString,'%s',1);
    
    Path=SonnetPath(aSonnetVersion);
else
    Path=SonnetPath();
end

% Call Sonnet to unlock the project
if isempty(findstr('-l',theOptions)) && isempty(findstr('-L',theOptions))
    aCallToSystem=['"' Path '\bin\soncmd.exe" -unlock "' theFilename '"'];
    [aStatus aMessage]=system(aCallToSystem);
    
    if aStatus
        error(['Failed to call soncmd to unlock the project: ' aMessage]);
    end
end

% If the 'C' flag was given delete the folder for the simulation to clean the data
if ~isempty(findstr('-c',theOptions)) || ~isempty(findstr('-C',theOptions)) % If they didnt pass an x value
    aProject=SonnetProject(theFilename);
    aProject.cleanProject;
end

% if the folder for simulation results doesn't exist create it
[~,result]=mkdir(['.\sondata\' strrep(theFilename,'.son','')]); %#ok<NASGU>

aStatus=[];
aMessage='';

% Prep System Call to invoke EM on the .son file
if isempty(findstr('-t',theOptions)) && isempty(findstr('-T',theOptions))
    % If we are drawing an EM Status window
    if ~isempty(findstr('-p',theOptions)) || ~isempty(findstr('-P',theOptions))
        % If we are going to start out paused
        aCallToSystem=['"' Path '\bin\emstatus.exe" "' theFilename '" &'];
    else
        % If we are running right away
        aCallToSystem=['"' Path '\bin\emstatus.exe" -Run "' theFilename '" &'];
    end
else
    % If we are going to do the simulation without a GUI
    aCallToSystem=['"' Path '\bin\em.exe" "' theFilename '"'];
end

% Make the call and wait for the simulation to complete successfully or to fail
if ~isempty(findstr('-t',theOptions)) || ~isempty(findstr('-T',theOptions))
    [aStatus aMessage]=system(aCallToSystem);
else
    isSimulationNotDone=true;
    aLogFileLocation=['sondata' filesep strrep(theFilename,'.son','') filesep 'log_errors.log'];
    
    % if the log file exists then delete it
    if ~isempty(dir(aLogFileLocation))
        warning off all
        delete(aLogFileLocation);
        warning on all
    end
    
    % Run the simulation
    system(aCallToSystem);
    
    while isSimulationNotDone
        
        % This pause needs to be here (cut it in half
        % or something if necessary but leave something there)
        % because otherwise we have full utilization
        % for of matlab while the simulation is running.
        pause(1);
        
        % If the log file doesnt exist yet then wait for it to exist.
        % If 5 seconds go by and there is no log file yet then
        % try again. If we have tried 3 times then throw an error.
        aFid=fopen(aLogFileLocation);
        aAmountOfTimeWaitedSoFar=0;
        aNumberOfSends=0;
        while aFid == -1
            aFid=fopen(aLogFileLocation);
            pause(1);
            aAmountOfTimeWaitedSoFar=aAmountOfTimeWaitedSoFar+1;
            if aAmountOfTimeWaitedSoFar > 5
                system(aCallToSystem);
                if aNumberOfSends==3
                    error('Sonnet never created a log file.')
                else
                    aNumberOfSends=aNumberOfSends+1;
                end
            end
        end
        
        % Search the file to see if we are done with the simulation
        while feof(aFid)==0
            aFileLine=fgetl(aFid);
            if ~isempty(findstr('Analysis completed',aFileLine))
                aStatus=0;
                aMessage='';
                isSimulationNotDone=false;
                break;
            end
            if ~isempty(findstr('Sonnet Error',aFileLine))
                iErrorCounter=1;
                fgetl(aFid);
                aFileLine=fgetl(aFid);
                while isempty(findstr('Date:',aFileLine))
                    aMessage{iErrorCounter}=aFileLine;
                    iErrorCounter=iErrorCounter+1;
                    aFileLine=fgetl(aFid);
                end
            end
            if ~isempty(findstr('Analysis failed',aFileLine))
                aStatus=1;
                isSimulationNotDone=false;
                break;
            end
        end
        fclose(aFid);
    end
end
