classdef SonnetGeometryComponent < handle
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % This class defines the values for an component
    % in the Geometry block of the Sonnet project file.
    %
    % SonnetLab, all included documentation, all included examples 
    % and all other files (unless otherwise specified) are copyrighted by Sonnet Software 
    % in 2010 with all rights reserved.
    %
    % THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS". ANY AND 
    % ALL EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED. UNDER NO CIRCUMSTANCES AND UNDER 
    % NO LEGAL THEORY, TORT, CONTRACT, OR OTHERWISE, SHALL THE COPYWRITE HOLDERS,  CONTRIBUTORS, 
    % MATLAB, OR SONNET SOFTWARE BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
    % CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF 
    % GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL 
    % DAMAGES OR LOSSES, OR FOR ANY DAMAGES EVEN IF THE COPYWRITE HOLDERS, CONTRIBUTORS, MATLAB, 
    % OR SONNET SOFTWARE HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES, OR FOR ANY CLAIM 
    % BY ANY OTHER PARTY.
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    properties
        
        LevelNumber
        Label
        ObjectId
        GroundReference
        TerminalWidthType
        TerminalWidthValue
        SchematicBoxLeftPosition
        SchematicBoxRightPosition
        SchematicBoxTopPosition
        SchematicBoxBottomPosition
        DisplayPackageSize
        PackageBoxLeftPosition
        PackageBoxRightPosition
        PackageBoxTopPosition
        PackageBoxBottomPosition
        PackageLength
        PackageWidth
        PackageHeight
        LabelPositionXCoordinate
        LabelPositionYCoordinate
        Type                        % Can be NONE, IDEAL, or SPARAM
        ArrayOfComponentPorts
        
        ReferencePlanes
        
        UnknownLines
        
        
    end
    
    methods
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function obj = SonnetGeometryComponent(theFid)
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % Defines the constructor for the SMD block.
            % The constructor will be passed the file ID from the
            %     SONNET GEO object constructor.
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            NumberOfComponentPorts=0;
            
            if nargin == 1              					%if we were passed 1 argument which means we go the theFid
                
                initialize(obj);
                
                %read in LevelNumber and Label
                obj.LevelNumber=fscanf(theFid,'%d',1);
                obj.Label=fscanf(theFid,'%s',1);
                
                while(1==1)
                    
                    % Read a string from the file,  we will use this to determine what property needs to be modified by using a case statement.
                    aTempString=fscanf(theFid,' %s',1); 		% Read a Value from the file, we will be using this to drive the switch statment
                    
                    switch aTempString
                        
                        case 'ID'
                            obj.ObjectId=fscanf(theFid,'%d',1);
                            
                        case 'GNDREF'
                            obj.GroundReference=fscanf(theFid,'%s',1);
                            
                        case 'TWTYPE'
                            obj.TerminalWidthType=fscanf(theFid,'%s',1);
                            
                        case 'TWVALUE'
                            obj.TerminalWidthValue=fscanf(theFid,'%f',1);
                            
                        case 'DRP1'
                            if isempty(obj.ReferencePlanes)                 % If we dont have a ReferencePlanes entry yet then make a new object for one
                                obj.ReferencePlanes=SonnetGeometryReferencePlane(theFid);
                            else                                            % If we already have an object for our ReferencePlanes entries then just add this one to the object using its add function
                                obj.ReferencePlanes.addNewSideFromFile(theFid);       % Tells the object to add a new Parallel subsection as defined from the file
                            end
                            
                        case 'SBOX'
                            obj.SchematicBoxLeftPosition=fscanf(theFid,'%f',1);
                            obj.SchematicBoxRightPosition=fscanf(theFid,'%f',1);
                            obj.SchematicBoxTopPosition=fscanf(theFid,'%f',1);
                            obj.SchematicBoxBottomPosition=fscanf(theFid,'%f',1);
                            
                        case 'PBSHW'
                            obj.DisplayPackageSize=fscanf(theFid,'%s',1);
                            
                        case 'PBOX'
                            obj.PackageBoxLeftPosition=fscanf(theFid,'%f',1);
                            obj.PackageBoxRightPosition=fscanf(theFid,'%f',1);
                            obj.PackageBoxTopPosition=fscanf(theFid,'%f',1);
                            obj.PackageBoxBottomPosition=fscanf(theFid,'%f',1);
                            
                        case 'PKG'
                            obj.PackageLength=fscanf(theFid,'%f',1);
                            obj.PackageWidth=fscanf(theFid,'%f',1);
                            obj.PackageHeight=fscanf(theFid,'%f',1);
                            
                        case 'LPOS'
                            obj.LabelPositionXCoordinate=fscanf(theFid,'%f',1);
                            obj.LabelPositionYCoordinate=fscanf(theFid,'%f',1);
                            
                        case 'TYPE'
                            aTempString=fscanf(theFid,'%s',1);
                            
                            if strcmp(aTempString,'IDEAL')==1
                                obj.Type.Type='IDEAL';
                                obj.Type.Idealtype=fscanf(theFid,'%s',1);
                                obj.Type.Compval=fscanf(theFid,'%s',1);
                                
                                % If Compval is a string for a 
                                % number then make it a number in memory
                                if ~isnan(str2double(obj.Type.Compval))
                                    obj.Type.Compval = str2double(obj.Type.Compval);
                                end
                                
                            elseif strcmp(aTempString,'SPARAM')==1
                                obj.Type.Type='SPARAM';
                                obj.Type.paramfileindex=fscanf(theFid,'%s',1);
                                
                            elseif strcmp(aTempString,'NONE')==1
                                obj.Type.Type='NONE';
                            end
                            
                        case 'SMDP'
                            NumberOfComponentPorts=NumberOfComponentPorts+1;
                            obj.ArrayOfComponentPorts{NumberOfComponentPorts}=SonnetGeometryComponentPort(theFid);
                            
                        case 'END'
                            break;
                            
                        otherwise																% If we dont recognize the line then we want to save it so we can write it out again.
                            obj.UnknownLines = [obj.UnknownLines aTempString fgetl(theFid) ' \n'];	% Add the line to the unknownlines array
                            
                    end
                end
                
            else
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                % we come here when we didn't recieve a file ID as an argument
                % which means that we are going to create a default object with
                % default values by calling the function's initialize method.
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                initialize(obj);
                
            end
            
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function initialize(obj)
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % This function initializes the SMD properties to some default
            %   values. This is called by the constructor and can
            %   be called by the user to reinitialize the object to
            %   default values.
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            aBackup=warning();
            warning off all
            aProperties = properties(obj);
            for iCounter = 1:length(aProperties)
                obj.(aProperties{iCounter}) = [];
            end
            warning(aBackup);
            
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function aNewObject=clone(obj)
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % This function builds a deep copy of this object
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            aNewObject=SonnetGeometryComponent();
            SonnetClone(obj,aNewObject);
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function writeObjectContents(obj, theFid, theVersion)
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % This function writes the values from the object to a file.
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            fprintf(theFid,'%s',obj.stringSignature(theVersion));
            
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function aSignature=stringSignature(obj,theVersion) %#ok<INUSD>
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % This function writes the values from the object to a string.
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            aSignature='SMD';
            
            if ~isempty(obj.LevelNumber)
                aSignature = [aSignature sprintf(' %d',obj.LevelNumber)];
            end
            
            if ~isempty(obj.Label)
                obj.Label=strrep(obj.Label,'"','');	% first get rid of all the quotation marks so that we dont have multiple sets of quotation marks in the string.
                obj.Label=['"' obj.Label '"'];		% add quotes to the label because the label must be wrapped in quotes.
                aSignature = [aSignature sprintf(' %s',obj.Label)];
            end
            
            aSignature = [aSignature sprintf('\n')];
            
            if ~isempty(obj.ObjectId)
                aSignature = [aSignature sprintf('ID %d\n',obj.ObjectId)];
            end
            
            if ~isempty(obj.GroundReference)
                aSignature = [aSignature sprintf('GNDREF %s\n',obj.GroundReference)];
            end
            
            if ~isempty(obj.TerminalWidthType)
                aSignature = [aSignature sprintf('TWTYPE %s\n',obj.TerminalWidthType)];
            end
            
            if ~isempty(obj.TerminalWidthValue)
                aSignature = [aSignature sprintf('TWVALUE %.15g\n',obj.TerminalWidthValue)];
            end
            
            if ~isempty(obj.ReferencePlanes)
                aSignature = [aSignature obj.ReferencePlanes.stringSignature()];
            end
            
            if ~isempty(obj.SchematicBoxLeftPosition)
                aSignature = [aSignature sprintf('SBOX %.15g',obj.SchematicBoxLeftPosition)];
                aSignature = [aSignature sprintf(' %.15g',obj.SchematicBoxRightPosition)];
                aSignature = [aSignature sprintf(' %.15g',obj.SchematicBoxTopPosition)];
                aSignature = [aSignature sprintf(' %.15g\n',obj.SchematicBoxBottomPosition)];
            end
            
            if ~isempty(obj.DisplayPackageSize)
                aSignature = [aSignature sprintf('PBSHW %s\n',obj.DisplayPackageSize)];
            end
            
            if ~isempty(obj.PackageBoxLeftPosition)
                aSignature = [aSignature sprintf('PBOX %.15g',obj.PackageBoxLeftPosition)];
                aSignature = [aSignature sprintf(' %.15g',obj.PackageBoxRightPosition)];
                aSignature = [aSignature sprintf(' %.15g',obj.PackageBoxTopPosition)];
                aSignature = [aSignature sprintf(' %.15g\n',obj.PackageBoxBottomPosition)];
            end
            
            if ~isempty(obj.PackageLength)
                aSignature = [aSignature sprintf('PKG %.15g',obj.PackageLength)];
                aSignature = [aSignature sprintf(' %.15g',obj.PackageWidth)];
                aSignature = [aSignature sprintf(' %.15g\n',obj.PackageHeight)];
            end
            
            if ~isempty(obj.LabelPositionXCoordinate) && ~isempty(obj.LabelPositionYCoordinate)
                aSignature = [aSignature sprintf('LPOS %.15g',obj.LabelPositionXCoordinate)];
                aSignature = [aSignature sprintf(' %.15g\n',obj.LabelPositionYCoordinate)];
            end
            
            if ~isempty(obj.Type)
                if strcmp(obj.Type.Type,'IDEAL')==1
                    if isa(obj.Type.Compval,'char')
                        aSignature = [aSignature sprintf('TYPE IDEAL %s %s\n',obj.Type.Idealtype,obj.Type.Compval)];
                    else
                        aSignature = [aSignature sprintf('TYPE IDEAL %s %d\n',obj.Type.Idealtype,obj.Type.Compval)];
                    end
                    
                elseif strcmp(obj.Type.Type,'SPARAM')==1
                    aSignature = [aSignature sprintf('TYPE SPARAM %s\n',obj.Type.paramfileindex)];
                    
                elseif strcmp(obj.Type.Type,'NONE')==1
                    aSignature = [aSignature sprintf('TYPE NONE\n')];
                    
                end
            end
            
            for iCounter= 1:length(obj.ArrayOfComponentPorts)
                aSignature = [aSignature obj.ArrayOfComponentPorts{iCounter}.stringSignature()]; %#ok<AGROW>
            end
            
            if (~isempty(obj.UnknownLines))
                aSignature = [aSignature strrep(obj.UnknownLines,'\n',sprintf('\n'))];
            end
            
            aSignature = [aSignature sprintf('END\n')];
            
        end
        
    end
    
end

